/*-------------------------------------------------------------------------+
| Copyright 2021 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.reuse.ui.view.fx;

import static java.util.Collections.emptyList;
import static org.fortiss.tooling.ext.reuse.utils.ReuseLibraryUtils.getAllLocalReuseLibraries;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.BasicEObjectImpl;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeContentProviderBase;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeTableUIProviderBase;
import org.fortiss.tooling.common.ui.javafx.control.treetableview.DynamicTreeTableViewer;
import org.fortiss.tooling.common.ui.javafx.layout.CompositeFXControllerBase;
import org.fortiss.tooling.ext.reuse.model.ReuseLibrary;

import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.SplitPane;
import javafx.scene.control.TextField;
import javafx.scene.control.TreeTableView;

/**
 * FX Controller for the {@link ReuseLibrary} view.
 *
 * @author bergemann
 */
@SuppressWarnings("unchecked")
public class ReuseLibraryFXController extends CompositeFXControllerBase<SplitPane, Node> {

	/** The {@link TreeTableView} for the {@link ReuseLibrary} table. */
	@FXML
	private TreeTableView<EObject> reuseLibraryTreeTableView;

	/** The refresh {@link Button} to refresh/load the {@link ReuseLibrary}s. */
	@FXML
	private Button refreshButton;

	/** The central top {@link Label} to display short messages. */
	@FXML
	private Label topLabel;

	/**
	 * The search {@link TextField} to search within the found
	 * {@link ReuseLibrary}s.
	 */
	@FXML
	private TextField textSearch;

	/**
	 * The {@link DynamicTreeTableViewer} to manage the {@link ReuseLibrary} view
	 * where the found {@link ReuseLibrary} and their reuse elements are shown in a
	 * table format.
	 */
	private DynamicTreeTableViewer<EObject> treeTableViewer;

	/**
	 * The {@link LibraryContainer} as dummy container for all
	 * {@link ReuseLibrary}s.
	 */
	private LibraryContainer root = new LibraryContainer();

	/** Time format when displaying updating time. */
	private final DateTimeFormatter timeFormat = DateTimeFormatter.ofPattern("HH:mm:ss");

	/** {@inheritDoc} */
	@Override
	public String getFXMLLocation() {
		return "ReuseLibraryViewFx.fxml";
	}

	/** {@inheritDoc} */
	@Override
	public void initialize() {
		setupLibraryTreeTable();
	}

	/** Generates the {@link ReuseLibrary} tree table. */
	private void setupLibraryTreeTable() {
		DynamicTreeContentProviderBase<EObject> contentProvider = new ReuseLibraryFXContentProvider();
		DynamicTreeTableUIProviderBase<EObject> uiProvider = new ReuseLibraryFXTreeTableUIProvider(this);

		textSearch.textProperty().addListener((obs, oldVal, newVal) -> {
			contentProvider.setFilterExpression(newVal);
			safelyUpdateViewer();
		});

		boolean showRoot = false; // do not show dummy container
		int revealLevel = 0; // all libraries collapsed
		treeTableViewer = new DynamicTreeTableViewer<>(reuseLibraryTreeTableView, root, showRoot, revealLevel,
				contentProvider, uiProvider);
		treeTableViewer.addColumn("Reuse Library Name", 200);
		treeTableViewer.addColumn("Reuse Element Name", 200);
		treeTableViewer.addColumn("UUID", 170);
		treeTableViewer.addColumn("Last Update", 120);
		treeTableViewer.addColumn("Comment", 300);

		boolean librariesFound = updateRootContainer();
		if (librariesFound) {
			updateLibraryTreeTable();
		}
	}

	/**
	 * Triggers the update of the {@link ReuseLibrary} tree table with the latest
	 * changes or sets it up if it does not exist yet.
	 */
	private void safelyUpdateViewer() {
		if (treeTableViewer == null) {
			setupLibraryTreeTable();
		} else {
			treeTableViewer.update();
		}
	}

	/**
	 * Triggers the update of the {@link ReuseLibrary} tree table with the latest
	 * changes.
	 */
	private void updateLibraryTreeTable() {
		safelyUpdateViewer();
		LocalDateTime currentTime = LocalDateTime.now();
		topLabel.setText("Library view was updated at " + timeFormat.format(currentTime));
	}

	/**
	 * Updates the root for all {@link ReuseLibrary}s that is a
	 * {@link LibraryContainer}. Returns true if at least one {@link ReuseLibrary}
	 * could be found, otherwise false.
	 */
	private boolean updateRootContainer() {
		List<ReuseLibrary> libraries = getAllLocalReuseLibraries();
		if (libraries == null || libraries.isEmpty()) {
			root.libraries = emptyList();
			safelyUpdateViewer();
			topLabel.setText("No library was found in the local folder.");
			return false;
		}
		root.libraries = libraries;
		return true;
	}

	/**
	 * Triggers the update of the whole {@link ReuseLibrary} view with the latest
	 * changes.
	 */
	public void updateReuseLibraryView() {
		boolean librariesFound = updateRootContainer();
		if (librariesFound) {
			updateLibraryTreeTable();
		}
	}

	/**
	 * Is triggered when the refresh button is pressed. Refreshes
	 * {@link ReuseLibrary}s/view.
	 */
	public void onRefreshButtonSubmit() {
		updateReuseLibraryView();
	}

	/**
	 * Local class to represent the (invisible) root/container for the tree table
	 * containing all found {@link ReuseLibrary}s.
	 */
	protected final class LibraryContainer extends BasicEObjectImpl {

		/** The list of {@link ReuseLibrary}s to be displayed. */
		List<ReuseLibrary> libraries = new ArrayList<>();
	}

}
